@@ -1,3 +1,5 @@
+import warnings
+
 from . import mock, BaseRedmineTestCase, Redmine
 
 from redminelib import engines, resultsets, exceptions
@@ -10,19 +12,17 @@ class RedmineTestCase(BaseRedmineTestCase):
         self.assertEqual(self.redmine.date_format, '%Y-%m-%d')
         self.assertEqual(self.redmine.datetime_format, '%Y-%m-%dT%H:%M:%SZ')
         self.assertEqual(self.redmine.raise_attr_exception, True)
-        self.assertEqual(self.redmine.resource_paths, ())
         self.assertEqual(self.redmine.engine.__class__, engines.DefaultEngine)
 
     def test_set_attributes_through_kwargs(self):
         FooEngine = type('FooEngine', (engines.BaseEngine,), {'create_session': lambda obj, **kwargs: None})
         redmine = Redmine(self.url, version='1.0', date_format='format', datetime_format='format',
-                          raise_attr_exception=False, resource_paths='foo.bar.baz', engine=FooEngine)
+                          raise_attr_exception=False, engine=FooEngine)
         self.assertEqual(redmine.url, self.url)
         self.assertEqual(redmine.ver, '1.0')
         self.assertEqual(redmine.date_format, 'format')
         self.assertEqual(redmine.datetime_format, 'format')
         self.assertEqual(redmine.raise_attr_exception, False)
-        self.assertEqual(redmine.resource_paths, 'foo.bar.baz')
         self.assertEqual(redmine.engine.__class__, FooEngine)
 
     def test_engine_class_exception(self):
@@ -61,8 +61,18 @@ class RedmineTestCase(BaseRedmineTestCase):
     @mock.patch('redminelib.open', mock.mock_open(), create=True)
     def test_successful_file_upload(self):
         self.response.status_code = 201
-        self.response.json.return_value = {'upload': {'token': '123456'}}
-        self.assertEqual(self.redmine.upload('foo'), '123456')
+        self.response.json.return_value = {'upload': {'id': 1, 'token': '123456'}}
+        self.assertEqual(self.redmine.upload('foo')['token'], '123456')
+
+    def test_successful_filestream_upload(self):
+        from io import StringIO
+        self.response.status_code = 201
+        self.response.json.return_value = {'upload': {'id': 1, 'token': '456789'}}
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always')
+            self.assertEqual(self.redmine.upload(StringIO(b'\xcf\x86oo'.decode('utf-8')))['token'], '456789')
+            self.assertEquals(len(w), 1)
+            self.assertIs(w[0].category, exceptions.PerformanceWarning)
 
     @mock.patch('redminelib.open', mock.mock_open(), create=True)
     def test_successful_file_download(self):
@@ -82,6 +92,10 @@ class RedmineTestCase(BaseRedmineTestCase):
     def test_file_upload_no_file_exception(self):
         self.assertRaises(exceptions.NoFileError, lambda: self.redmine.upload('foo',))
 
+    def test_file_upload_file_object_exception(self):
+        f = type('FileObject', (), {'close': lambda obj: None})()
+        self.assertRaises(exceptions.FileObjectError, lambda: self.redmine.upload(f))
+
     def test_file_upload_not_supported_exception(self):
         self.redmine.ver = '1.0.0'
         self.assertRaises(exceptions.VersionMismatchError, lambda: self.redmine.upload('foo',))
@@ -133,4 +147,3 @@ class RedmineTestCase(BaseRedmineTestCase):
         self.assertEqual(redmine.date_format, self.redmine.date_format)
         self.assertEqual(redmine.datetime_format, self.redmine.datetime_format)
         self.assertEqual(redmine.raise_attr_exception, self.redmine.raise_attr_exception)
-        self.assertEqual(redmine.resource_paths, self.redmine.resource_paths)
