@@ -98,14 +98,17 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.response.json.return_value = responses['project']['get']
         project = self.redmine.project.new()
         project.name = 'Foo'
-        self.assertEqual(project.save(), True)
+        self.assertIsInstance(project.save(), resources.Project)
+        self.assertEqual(project.id, 1)
+        project = self.redmine.project.new().save(name='Foo')
+        self.assertIsInstance(project, resources.Project)
         self.assertEqual(project.id, 1)
 
     def test_saving_existing_resource_updates_it(self):
         self.response.json.return_value = responses['project']['get']
         project = self.redmine.project.get(1)
         project.name = 'Bar'
-        self.assertEqual(project.save(), True)
+        self.assertIsInstance(project.save(), resources.Project)
         self.response.json.return_value = {'project': {'id': 1, 'name': 'Bar'}}
         project.refresh()
         self.assertEqual(project.name, 'Bar')
@@ -134,10 +137,11 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
 
     def test_bulk_decode(self):
         from datetime import date, datetime
-        encoded = {'start_date': date(2014, 3, 9), 'created_at': datetime(2014, 3, 9, 20, 2, 2)}
+        encoded = {'start_date': date(2014, 3, 9), 'created_at': datetime(2014, 3, 9, 20, 2, 2), 'include': ['a', 'b']}
         decoded = self.redmine.project.resource_class.bulk_decode(encoded, self.redmine.project)
         self.assertEqual(decoded['start_date'], '2014-03-09')
         self.assertEqual(decoded['created_at'], '2014-03-09T20:02:02Z')
+        self.assertEqual(decoded['include'], 'a,b')
 
     def test_bulk_encode(self):
         from datetime import date, datetime
@@ -227,7 +231,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         project.homepage = 'http://foo.bar'
         project.parent_id = 3
         project.custom_fields = [{'id': 1, 'value': 'bar'}]
-        self.assertEqual(project.save(), True)
+        self.assertIsInstance(project.save(), resources.Project)
         self.assertEqual(project.custom_fields[0].value, 'bar')
 
     def test_project_relations(self):
@@ -240,6 +244,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.assertIsInstance(project.versions, resultsets.ResourceSet)
         self.assertIsInstance(project.news, resultsets.ResourceSet)
         self.assertIsInstance(project.issues, resultsets.ResourceSet)
+        self.assertIsInstance(project.files, resultsets.ResourceSet)
 
     def test_project_includes(self):
         response_includes = responses['project']['get']
@@ -254,6 +259,9 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         response_includes['project'].update({'enabled_modules': [{'id': 36, 'name': 'issue_tracking'}]})
         self.response.json.return_value = response_includes
         self.assertEqual(project.enabled_modules, ['issue_tracking'])
+        response_includes['project'].update({'time_entry_activities': [{'id': 1, 'name': 'developing'}]})
+        self.response.json.return_value = response_includes
+        self.assertEqual(project.time_entry_activities, [{'id': 1, 'name': 'developing'}])
 
     def test_project_returns_status_without_conversion(self):
         self.response.json.return_value = {'project': {'name': 'Foo', 'id': 1, 'status': 1}}
@@ -328,7 +336,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         issue.subject = 'Foo'
         issue.description = 'foobar'
         issue.custom_fields = [{'id': 1, 'value': 'bar'}]
-        self.assertEqual(issue.save(), True)
+        self.assertIsInstance(issue.save(), resources.Issue)
         self.assertEqual(issue.custom_fields[0].value, 'bar')
 
     def test_issue_relations(self):
@@ -420,6 +428,18 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         issue.version_id = 1
         self.assertEqual(issue.fixed_version.id, 1)
 
+    def test_issue_assigned_to_id_can_be_removed_via_none(self):
+        self.response.json.return_value = responses['issue']['get']
+        issue = self.redmine.issue.get(1)
+        issue.assigned_to_id = None
+        self.assertEqual(issue.assigned_to_id, '')
+
+    def test_issue_assigned_to_id_can_be_removed_via_zero(self):
+        self.response.json.return_value = responses['issue']['get']
+        issue = self.redmine.issue.get(1)
+        issue.assigned_to_id = 0
+        self.assertEqual(issue.assigned_to_id, '')
+
     def test_issue_is_new(self):
         issue = self.redmine.issue.new()
         self.assertEqual(int(issue), 0)
@@ -490,7 +510,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         time_entry.hours = 3
         time_entry.issue_id = 2
         time_entry.activity_id = 2
-        self.assertEqual(time_entry.save(), True)
+        self.assertIsInstance(time_entry.save(), resources.TimeEntry)
 
     def test_time_entry_translate_params(self):
         manager = self.redmine.time_entry
@@ -525,6 +545,12 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
     def test_enumeration_version(self):
         self.assertEqual(self.redmine.enumeration.resource_class.redmine_version, '2.2')
 
+    def test_enumeration_get(self):
+        self.response.json.return_value = responses['enumeration']['filter']
+        enumeration = self.redmine.enumeration.get(1, resource='time_entry_activities')
+        self.assertEqual(enumeration.id, 1)
+        self.assertEqual(enumeration.name, 'Foo')
+
     def test_enumeration_filter(self):
         self.response.json.return_value = responses['enumeration']['filter']
         enumerations = self.redmine.enumeration.filter(resource='time_entry_activities')
@@ -549,6 +575,12 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.assertEqual(attachment.id, 1)
         self.assertEqual(attachment.filename, 'foo.jpg')
 
+    def test_attachment_update(self):
+        self.response.json.return_value = responses['attachment']['get']
+        attachment = self.redmine.attachment.get(1)
+        attachment.filename = 'bar.jpg'
+        self.assertIsInstance(attachment.save(), resources.Attachment)
+
     def test_attachment_delete(self):
         self.response.json.return_value = responses['attachment']['get']
         attachment = self.redmine.attachment.get(1)
@@ -588,6 +620,80 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.response.iter_content = lambda chunk_size: (str(num) for num in range(0, 5))
         self.assertEqual(self.redmine.attachment.get(1).download('/some/path'), '/some/path/bar.txt')
 
+    def test_file_version(self):
+        self.assertEqual(self.redmine.file.resource_class.redmine_version, '3.4')
+
+    def test_file_get(self):
+        self.response.json.return_value = responses['attachment']['get']
+        f = self.redmine.file.get(1)
+        self.assertEqual(f.id, 1)
+        self.assertEqual(f.filename, 'foo.jpg')
+
+    def test_file_filter(self):
+        self.response.json.return_value = responses['file']['filter']
+        files = self.redmine.file.filter(project_id=1)
+        self.assertEqual(files[0].id, 1)
+        self.assertEqual(files[0].filename, 'foo.jpg')
+        self.assertEqual(files[1].id, 2)
+        self.assertEqual(files[1].filename, 'bar.jpg')
+
+    @mock.patch('os.path.isfile', mock.Mock())
+    @mock.patch('os.path.getsize', mock.Mock())
+    @mock.patch('redminelib.open', mock.mock_open(), create=True)
+    def test_file_create(self):
+        self.set_patch_side_effect([
+            mock.Mock(status_code=201, history=[], **{'json.return_value': {'upload': {'id': 1, 'token': '1.1234'}}}),
+            mock.Mock(status_code=200, history=[], content='')
+        ])
+        f = self.redmine.file.create(project_id=1, filename='foo.jpg', path='foo', return_complete=False)
+        self.assertEqual(f.id, 1)
+        self.assertRaises(exceptions.ResourceAttrError, lambda: f.filename)
+
+    def test_file_update(self):
+        self.response.json.return_value = responses['attachment']['get']
+        f = self.redmine.file.get(1)
+        f.filename = 'bar.jpg'
+        self.assertIsInstance(f.save(), resources.File)
+
+    def test_file_delete(self):
+        self.response.json.return_value = responses['attachment']['get']
+        f = self.redmine.file.get(1)
+        self.response.content = ''
+        self.assertEqual(f.delete(), True)
+        self.assertEqual(self.redmine.file.delete(1), True)
+
+    def test_file_custom_str(self):
+        self.response.json.return_value = responses['attachment']['get']
+        self.assertEqual(str(self.redmine.file.get(1)), 'foo.jpg')
+
+    def test_file_custom_str_without_filename(self):
+        self.response.json.return_value = responses['attachment']['get']
+        f = self.redmine.file.get(1)
+        del f['_decoded_attrs']['filename']
+        self.assertEqual(str(f), '1')
+
+    def test_file_custom_repr(self):
+        self.response.json.return_value = responses['attachment']['get']
+        self.assertEqual(repr(self.redmine.file.get(1)), '<redminelib.resources.File #1 "foo.jpg">')
+
+    def test_file_custom_repr_without_subject(self):
+        self.response.json.return_value = responses['attachment']['get']
+        f = self.redmine.file.get(1)
+        del f['_decoded_attrs']['filename']
+        self.assertEqual(repr(f), '<redminelib.resources.File #1>')
+
+    def test_file_url(self):
+        self.response.json.return_value = responses['attachment']['get']
+        self.assertEqual(self.redmine.file.get(1).url, '{0}/attachments/1'.format(self.url))
+
+    @mock.patch('redminelib.open', mock.mock_open(), create=True)
+    def test_file_download(self):
+        response = responses['attachment']['get']
+        response['attachment']['content_url'] = 'http://foo/bar.txt'
+        self.response.json.return_value = response
+        self.response.iter_content = lambda chunk_size: (str(num) for num in range(0, 5))
+        self.assertEqual(self.redmine.file.get(1).download('/some/path'), '/some/path/bar.txt')
+
     def test_wiki_page_version(self):
         self.assertEqual(self.redmine.wiki_page.resource_class.redmine_version, '2.2')
 
@@ -620,7 +726,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
             {'wiki_page': {'title': 'Foo', 'version': 1, 'created_on': '2012-06-27T12:48:15Z'}}
         wiki_page = self.redmine.wiki_page.get('Foo', project_id=1)
         wiki_page.text = 'Foo'
-        self.assertEqual(wiki_page.save(), True)
+        self.assertIsInstance(wiki_page.save(), resources.WikiPage)
         self.assertEqual(wiki_page.version, 2)
 
     def test_wiki_page_refresh_by_title(self):
@@ -717,7 +823,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.response.json.return_value = responses['project_membership']['get']
         membership = self.redmine.project_membership.get(1)
         membership.role_ids = [1, 2]
-        self.assertEqual(membership.save(), True)
+        self.assertIsInstance(membership.save(), resources.ProjectMembership)
         self.assertEqual(membership.roles[0].id, 1)
         self.assertEqual(membership.roles[1].id, 2)
 
@@ -773,7 +879,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.response.json.return_value = responses['issue_category']['get']
         category = self.redmine.issue_category.get(1)
         category.name = 'Bar'
-        self.assertEqual(category.save(), True)
+        self.assertIsInstance(category.save(), resources.IssueCategory)
 
     def test_issue_category_is_new(self):
         category = self.redmine.issue_category.new()
@@ -864,7 +970,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.response.json.return_value = responses['version']['get']
         version = self.redmine.version.get(1)
         version.name = 'Bar'
-        self.assertEqual(version.save(), True)
+        self.assertIsInstance(version.save(), resources.Version)
 
     def test_version_returns_status_without_conversion(self):
         self.response.json.return_value = {'version': {'id': 1, 'name': 'Foo', 'status': 'foo'}}
@@ -925,7 +1031,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         user = self.redmine.user.get(1)
         user.lastname = 'Foo'
         user.firstname = 'Bar'
-        self.assertEqual(user.save(), True)
+        self.assertIsInstance(user.save(), resources.User)
 
     def test_user_custom_str(self):
         self.response.json.return_value = responses['user']['get']
@@ -1001,7 +1107,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.response.json.return_value = responses['group']['get']
         group = self.redmine.group.get(1)
         group.name = 'Bar'
-        self.assertEqual(group.save(), True)
+        self.assertIsInstance(group.save(), resources.Group)
 
     def test_group_includes(self):
         response_includes = responses['group']['get']
@@ -1060,6 +1166,12 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
     def test_news_version(self):
         self.assertEqual(self.redmine.news.resource_class.redmine_version, '1.1')
 
+    def test_news_get(self):
+        self.response.json.return_value = responses['news']['all']
+        news = self.redmine.news.get(1)
+        self.assertEqual(news.id, 1)
+        self.assertEqual(news.title, 'Foo')
+
     def test_news_all(self):
         self.response.json.return_value = responses['news']['all']
         news = self.redmine.news.all()
@@ -1097,6 +1209,12 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
     def test_issue_status_version(self):
         self.assertEqual(self.redmine.issue_status.resource_class.redmine_version, '1.3')
 
+    def test_issue_status_get(self):
+        self.response.json.return_value = responses['issue_status']['all']
+        status = self.redmine.issue_status.get(1)
+        self.assertEqual(status.id, 1)
+        self.assertEqual(status.name, 'Foo')
+
     def test_issue_status_all(self):
         self.response.json.return_value = responses['issue_status']['all']
         statuses = self.redmine.issue_status.all()
@@ -1112,6 +1230,12 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
     def test_tracker_version(self):
         self.assertEqual(self.redmine.tracker.resource_class.redmine_version, '1.3')
 
+    def test_tracker_get(self):
+        self.response.json.return_value = responses['tracker']['all']
+        tracker = self.redmine.tracker.get(1)
+        self.assertEqual(tracker.id, 1)
+        self.assertEqual(tracker.name, 'Foo')
+
     def test_tracker_all(self):
         self.response.json.return_value = responses['tracker']['all']
         trackers = self.redmine.tracker.all()
@@ -1127,6 +1251,12 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
     def test_query_version(self):
         self.assertEqual(self.redmine.query.resource_class.redmine_version, '1.3')
 
+    def test_query_get(self):
+        self.response.json.return_value = responses['query']['all']
+        query = self.redmine.query.get(1)
+        self.assertEqual(query.id, 1)
+        self.assertEqual(query.name, 'Foo')
+
     def test_query_all(self):
         self.response.json.return_value = responses['query']['all']
         queries = self.redmine.query.all()
@@ -1142,6 +1272,12 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
     def test_custom_field_version(self):
         self.assertEqual(self.redmine.custom_field.resource_class.redmine_version, '2.4')
 
+    def test_custom_field_get(self):
+        self.response.json.return_value = responses['custom_field']['all']
+        field = self.redmine.custom_field.get(1)
+        self.assertEqual(field.id, 1)
+        self.assertEqual(field.name, 'Foo')
+
     def test_custom_field_all(self):
         self.response.json.return_value = responses['custom_field']['all']
         fields = self.redmine.custom_field.all()
